<?php

namespace FiloBlu\Cegid\Helper\Cegid;

use DateTime;
use FiloBlu\CegidY2\Api\CustomerWcfService\ServiceType\Add;
use FiloBlu\CegidY2\Api\CustomerWcfService\ServiceType\Search;
use FiloBlu\CegidY2\Api\CustomerWcfService\ServiceType\Update;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\AddNewCustomer;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\AddNewCustomerFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\AddressDataTypeFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\BirthDateDataTypeFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\CustomerInputData;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\CustomerInputDataFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\CustomerInsertData;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\CustomerInsertDataFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\CustomerSearchDataTypeFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\EmailDataTypeFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\RetailContext;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\RetailContextFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\SearchCustomerIdsFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\SearchCustomerIdsResponse;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\UpdateCustomer;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\UpdateCustomerFactory;
use FiloBlu\CegidY2\Api\CustomerWcfService\StructType\UpdateCustomerResponse;
use FiloBlu\CegidY2\Service\CredentialProvider;
use FiloBlu\CegidY2\Service\ServiceProvider;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Magento\Customer\Api\Data\CustomerInterfaceFactory;
use Magento\Directory\Model\ResourceModel\Country\CollectionFactory;
use Magento\Framework\Exception\InputException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\State\InputMismatchException;
use RuntimeException;

use FiloBlu\Cegid\Helper\Cegid\Base as BaseHelper;

/**
 * Class Customer
 * @package FiloBlu\Cegid\Helper\Cegid
 */
class Customer
{
    /**
     * @var int
     */
    const CUSTOMER_INDIVIDUAL = 0;

    /**
     * @var int
     */
    const CUSTOMER_COMPANY = 1;

    /**
     * @var EmailDataTypeFactory
     */
    protected $emailDataTypeFactory;
    /**
     * @var ServiceProvider
     */
    protected $searchFactory;
    /**
     * @var CustomerSearchDataTypeFactory
     */
    protected $searchDataFactory;
    /**
     * @var SearchCustomerIdsFactory
     */
    protected $searchCustomerIdsFactory;
    /**
     * @var CredentialProvider
     */
    protected $credentialProvider;
    /**
     * @var RetailContextFactory
     */
    protected $retailContextFactory;
    /**
     * @var ServiceProvider
     */
    protected $addFactory;
    /**
     * @var AddNewCustomerFactory
     */
    protected $addNewCustomerFactory;
    /**
     * @var CustomerInsertDataFactory
     */
    protected $customerInsertDataFactory;
    /**
     * @var BirthDateDataTypeFactory
     */
    protected $birthDateDataFactory;
    /**
     * @var AddressDataTypeFactory
     */
    protected $addressDataTypeFactory;
    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var CollectionFactory
     */
    protected $countryCollectionFactory;
    /**
     * @var ServiceProvider
     */
    protected $updateFactory;
    /**
     * @var UpdateCustomerFactory
     */
    protected $updateCustomerFactory;
    /**
     * @var CustomerInputDataFactory
     */
    protected $customerInputDataFactory;
    /**
     * @var BaseHelper
     */
    protected $baseHelper;
    /**
     * @var CustomerInterfaceFactory
     */
    protected $customerInterfaceFactory;

    /**
     * Customer constructor.
     * @param AddNewCustomerFactory $addNewCustomerFactory
     * @param AddressDataTypeFactory $addressDataTypeFactory
     * @param EmailDataTypeFactory $emailDataTypeFactory
     * @param BirthDateDataTypeFactory $birthDateDataFactory
     * @param CustomerSearchDataTypeFactory $searchDataTypeFactory
     * @param CustomerInsertDataFactory $customerInsertDataFactory
     * @param CustomerRepositoryInterface $customerRepository
     * @param SearchCustomerIdsFactory $searchCustomerIdsFactory
     * @param UpdateCustomerFactory $updateCustomerFactory
     * @param CustomerInputDataFactory $customerInputDataFactory
     * @param RetailContextFactory $retailContextFactory
     * @param CredentialProvider $credentialProvider
     * @param ServiceProvider $addFactory
     * @param ServiceProvider $searchFactory
     * @param ServiceProvider $updateFactory
     * @param CollectionFactory $countryCollectionFactory
     * @param Base $baseHelper
     * @param CustomerInterfaceFactory $customerInterfaceFactory
     */
    public function __construct(
        AddNewCustomerFactory $addNewCustomerFactory,
        AddressDataTypeFactory $addressDataTypeFactory,
        EmailDataTypeFactory $emailDataTypeFactory,
        BirthDateDataTypeFactory $birthDateDataFactory,
        CustomerSearchDataTypeFactory $searchDataTypeFactory,
        CustomerInsertDataFactory $customerInsertDataFactory,
        CustomerRepositoryInterface $customerRepository,
        SearchCustomerIdsFactory $searchCustomerIdsFactory,
        UpdateCustomerFactory $updateCustomerFactory,
        CustomerInputDataFactory $customerInputDataFactory ,
        RetailContextFactory $retailContextFactory,
        CredentialProvider $credentialProvider,
        ServiceProvider $addFactory,
        ServiceProvider $searchFactory,
        ServiceProvider $updateFactory,
        CollectionFactory $countryCollectionFactory,
        BaseHelper $baseHelper,
        CustomerInterfaceFactory $customerInterfaceFactory

    )
    {
        $this->addressDataTypeFactory = $addressDataTypeFactory;
        $this->emailDataTypeFactory = $emailDataTypeFactory;
        $this->searchFactory = $searchFactory;
        $this->searchDataFactory = $searchDataTypeFactory;
        $this->searchCustomerIdsFactory = $searchCustomerIdsFactory;
        $this->credentialProvider = $credentialProvider;
        $this->retailContextFactory = $retailContextFactory;
        $this->addFactory = $addFactory;
        $this->addNewCustomerFactory = $addNewCustomerFactory;
        $this->customerInsertDataFactory = $customerInsertDataFactory;
        $this->birthDateDataFactory = $birthDateDataFactory;
        $this->customerRepository = $customerRepository;
        $this->countryCollectionFactory = $countryCollectionFactory;
        $this->updateFactory = $updateFactory;
        $this->updateCustomerFactory = $updateCustomerFactory;
        $this->customerInputDataFactory  = $customerInputDataFactory;
        $this->baseHelper = $baseHelper;
        $this->customerInterfaceFactory = $customerInterfaceFactory;
    }

    /**
     * @param CustomerInterface $customer
     * @return SearchCustomerIdsResponse
     *
     */
    public function searchCustomerByEmail(CustomerInterface $customer)
    {
        /** @var Search $searchService */
        $searchService = $this->searchFactory->create();

        $emailData = $this->emailDataTypeFactory->create(['email' => $customer->getEmail()]);

        $searchData = $this->searchDataFactory->create();
        $searchData->setEmailData($emailData);

        $parameters = $this->searchCustomerIdsFactory->create();
        $parameters->setSearchData($searchData);


        $parameters->setClientContext($this->getCustomerRetailContext());

        $searchService->SearchCustomerIds($parameters);

        $request = $searchService->getLastRequest();
        $response = $searchService->getLastResponse();
        $result = $searchService->getResult();

        $this->baseHelper->writeLog('searchCustomerByEmail',$request,$response);

        if ($result !== false) {
            // This will have eventually the cedig code ID we need
            return $result;
        }

        throw new RuntimeException(implode(PHP_EOL, $searchService->getLastError()));
    }

    /**
     * @return RetailContext
     */
    public function getCustomerRetailContext()
    {
        return $this->retailContextFactory->create([
            'databaseId' => $this->credentialProvider->getDatabase('customer_wcf_service')
        ]);
    }

    /**
     * @param CustomerInterface $customer
     * @return CustomerInterface
     * @throws InputException
     * @throws LocalizedException
     * @throws InputMismatchException
     */
    public function createCustomer(CustomerInterface $customer)
    {
        $emailData = $this->emailDataTypeFactory->create(['email' => $customer->getEmail()]);

        /** @var CustomerInsertData $customerData */

        $customerData = $this->customerInsertDataFactory->create();

        $customerData->setEmailData($emailData);
        $customerData->setFirstName($customer->getFirstname());
        $customerData->setLastName($customer->getLastname());
        $customerData->setIsCompany(false);
        if (($sex = $customer->getGender()) !== null) {
            switch ((int)$sex) {
                case 1:
                    $customerData->setSex('M');
                    break;
                case 2:
                    $customerData->setSex('F');
                    break;
            }
        }

        if (($dob = $customer->getDob()) !== null) {
            $birthDate = DateTime::createFromFormat('Y-m-d', $dob);
            if ($birthDate !== false) {
                $customerData->setBirthDateData($this->birthDateDataFactory->create([
                        'birthDateDay' => $birthDate->format('d'),
                        'birthDateMonth' => $birthDate->format('m'),
                        'birthDateYear' => $birthDate->format('Y')
                    ]
                ));
            }
        }

        $addresses = $customer->getAddresses();
        if ($addresses !== null && count($addresses) > 0) {
            $address = $addresses[0];
            $iso3CountryCode = $this->countryCollectionFactory->create()
                ->addFieldToFilter('country_id', $address->getCountryId())
                ->getFirstItem()
                ->getData('iso3_code');
            $customerData->setAddressData($this->addressDataTypeFactory->create([
                'addressLine1' => $this->baseHelper->getAddressStreet($address,1),
                'addressLine2' => $this->baseHelper->getAddressStreet($address,2),
                'addressLine3' => $this->baseHelper->getAddressStreet($address,3),
                'brandName' => null,
                //'city' => $address->getCity(),
                'countryId' => $iso3CountryCode,
                //'countryIdType' => 'Internal', //this is not required
                'nata' => null,
                //'regionId' => $address->getRegion()->getRegionCode(),
                'zipCode' => $address->getPostcode()
            ]));
        }

        /** @var Add $addService */
        $addService = $this->addFactory->create();

        /** @var AddNewCustomer $parameters */
        $parameters = $this->addNewCustomerFactory->create();
        $parameters->setCustomerData($customerData);
        $parameters->setClientContext($this->getCustomerRetailContext());

        $addService->AddNewCustomer($parameters);

        $request = $addService->getLastRequest();
        $response = $addService->getLastResponse();
        $result = $addService->getResult();

        $this->baseHelper->writeLog('createCustomer',$request,$response);

        if ($result !== false) {
            $result = $addService->getResult()->getAddNewCustomerResult();
            $customer->setCustomAttribute('cegid_id', $result);
            if ($customer->getId()) {
                $this->customerRepository->save($customer);
            }
            return $customer;
        }

        throw new RuntimeException(implode(PHP_EOL, $addService->getLastError()));
    }

    /**
     * @param CustomerInterface $customer
     * @return UpdateCustomerResponse
     */
    public function updateCustomer(CustomerInterface $customer)
    {
        /** @var Update $updateService */
        $updateService = $this->updateFactory->create();
        /** @var  UpdateCustomer $parameters */
        $parameters = $this->updateCustomerFactory->create();

        /** @var  CustomerInputData $customerData */
        $customerData = $this->customerInputDataFactory->create();

        $emailData = $this->emailDataTypeFactory->create(['email' => $customer->getEmail()]);
        $customerData->setEmailData($emailData);
        $customerData->setFirstName($customer->getFirstname());
        $customerData->setLastName($customer->getLastname());
        $customerData->setIsCompany(false);
        if (($sex = $customer->getGender()) !== null) {
            switch ((int)$sex) {
                case 1:
                    $customerData->setSex('M');
                    break;
                case 2:
                    $customerData->setSex('F');
                    break;
            }
        }

        if (($dob = $customer->getDob()) !== null) {
            $birthDate = DateTime::createFromFormat('Y-m-d', $dob);
            if ($birthDate !== false) {
                $customerData->setBirthDateData($this->birthDateDataFactory->create([
                        'birthDateDay' => $birthDate->format('d'),
                        'birthDateMonth' => $birthDate->format('m'),
                        'birthDateYear' => $birthDate->format('Y')
                    ]
                ));
            }
        }

        if (($addresses = $customer->getAddresses()) !== null &&   ($address = $addresses[0])!==null) {

            $iso3CountryCode = $this->countryCollectionFactory->create()
                ->addFieldToFilter('country_id', $address->getCountryId())
                ->getFirstItem()
                ->getData('iso3_code');
            $customerData->setAddressData($this->addressDataTypeFactory->create([
                'addressLine1' => $this->baseHelper->getAddressStreet($address,1),
                'addressLine2' => $this->baseHelper->getAddressStreet($address,2),
                'addressLine3' => $this->baseHelper->getAddressStreet($address,3),
                'brandName' => null,
                //'city' => $address->getCity(),
                'countryId' => $iso3CountryCode,
                'countryIdType' => 'Internal',
                'nata' => null,
                //'regionId' => $address->getRegionId(),
                'zipCode' => $address->getPostcode()
            ]));
        }


        $parameters->setModifiedData($customerData);
        $parameters->setCustomerId($customer->getCustomAttribute('cegid_id'));
        $parameters->setClientContext($this->getCustomerRetailContext());
        if ($updateService->UpdateCustomer($parameters) !== false) {
            return $updateService->getResult();
        }

        throw new RuntimeException(implode(PHP_EOL, $updateService->getLastError()));
    }

    public function getAddressStreet($address,$index = null){
        $result = array();
        $street_data = $address->getStreet();
        if (is_array($street_data)){
            $result[1] = (isset($street_data[0]) ? $street_data[0] : "");
            $result[2] = (isset($street_data[1]) ? $street_data[1] : "");
            $result[3] = (isset($street_data[2]) ? $street_data[2] : "");
        } else {
            return $street_data;
        }

        if ($index){
            return $result[$index];
        }

        return $result;
    }

    /**
     * @return CustomerInterface
     */
    public function customerInterfaceFactoryCreate(): CustomerInterface
    {
        return $this->customerInterfaceFactory->create();
    }

}